!!! info ""
:construction: Section under Construction :construction:

This section will allow you to launch a proper local environment of the Ecolyo application

## Initialisation of the project

### Clone the project repository

Use following command in your favorite terminal to clone the projet:

```
git clone https://forge.grandlyon.com/web-et-numerique/llle_project/ecolyo.git
```

### User settings

Open the project with VSCode and add the following code into .vscode/settings.json:

```
{
  "workbench.colorCustomizations": {
     /* Put anything you like here */
  },
  "editor.snippetSuggestions": "top",
  "editor.tabSize": 2,
  "editor.formatOnSave": true,
  "diffEditor.ignoreTrimWhitespace": false,
  "window.zoomLevel": 0,
  "gitlens.advanced.messages": {
      "suppressShowKeyBindingsNotice": true
  },
  "breadcrumbs.enabled": true,
  "workbench.settings.editor": "json",
  "editor.tabSize": 2,
  "editor.formatOnSave": true,
  "editor.formatOnPaste": true
}
```

If this file is not empty, you can simply add the follwing lines to it:

```
  "gitlens.advanced.messages": {
    "suppressShowKeyBindingsNotice": true
  },
  "breadcrumbs.enabled": true,
  "workbench.settings.editor": "json",
  "editor.formatOnSave": true,
  "diffEditor.ignoreTrimWhitespace": false,
  "window.zoomLevel": 0,
```

## Installation of the projet

### Installation of node modules

To install required node module, open a terminal on the root folder of the projet and type the following command:

```
yarn install
```

### Start Ecolyo application

To start ecolyo application just launch the command:

```
yarn start
```

If the application starts successfully you should obtain the following message:

> App successfully compiled!
> Dev assets:http://localhost:8888

Once the application is started, run the easy-cozy stack with docker. 

!!! info "Fix webpack error on cozy script"
    **Deprecated** Updated cozy-script lib
    To fix it launch the following command in node_modules/cozy-script folder:
    ```
    yarn add webpack
    ```

### Run easy-cozy dev stack

The dev stack will contain the cozy-stack with some default applications and konnectors and allow you to run this stack locally.

First login to the forge registry:

```
docker login registry.forge.grandlyon.com
```

You will need to enter for credentials.

Next pull the last image of the dev stack:

```
docker pull registry.forge.grandlyon.com/web-et-numerique/llle_project/ecolyo/cozy-env:latest
```

#### On windows

Launch the stack will following command:

```
yarn winstack
```

A Powershell window will open. Wait until the success message:
> Everything is setup. Go to http://ecolyo.cozy.tools:8080/
> To exit, press ^C

Once successfull you can launch your local cozy with following url: [Cozy local url](http://cozy.tools:8080/)
Local database can also be consulted at the following url: [Cozy local DB](http://localhost:5984/_utils/)

#### On Linux

From the project folder, launch following command:

```
yarn linuxstack
```

Wait until the success message:
> Everything is setup. Go to http://ecolyo.cozy.tools:8080/
> To exit, press ^C

Once successfull you can launch your local cozy with following url [Cozy local url](http://cozy.tools:8080/) and connect with the password '**cozy**'.
Local database can also be consulted at the following url: [Cozy local DB](http://localhost:5984/_utils/)

## Import moke data

!!! info "Requirement"
Your dev stack should be running to import the moke data

First install ACH using yarn :

```
yarn global add cozy-ach
```

Next launch the script **createDayDataFiles.js** available into **scripts** folder

```
cd scripts
node createDayDataFiles.js
```

!!! info ""
You can change the period of data generated by changing the following lines:

    ```
    const startingdate = DateTime.local()
        .plus({ days: -120 })
        .startOf('day')
    const endingDate = DateTime.local()
        .plus({ days: -1 })
        .startOf('day')
    ```

    Just replace -120 by the value needed

Json files with data will be created into **scripts/data** folder.
ACH script "importData.bat" will load the content of the file into your couchDB (database of your dev stack). Launch it using following commands:

```
cd scripts
./importData.bat
```

!!! info ""
You'll be prompted for a cozysessid, you can get it in the web console ; in the 'application' tab, look for the cookies and just copy the cozysessid value.

A new browser windows will open and asking you approval to import data. Just click on **Approve**

Finally your data is loaded in the Ecolyo Application !

## Simulate konnectors connection

!!! info "Requirement"
Your dev stack should be running to import data needed to simulate konnectors connection

When using local environment, konnectors are not able to connect.
To simulate the connection you can use the script **createConnection.js** available in **scripts** folder.

First update the Bearer and cozysessid information from the script:
```
  Authorization:
    'Bearer xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx',
  Cookie: 'cozysessid=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx',
```
To obtain your own Bearer and cozysessid you need to have the application launched and display the web console (F12) in your navigator and select the Network tab. Just select one request and retrieve information for Request Header:

![Bearer&cozysessid](bearer_cozyssesid.png)

Next launch the script :

```
cd scripts
node createConnection.js
```

Your data is now loaded and you can refresh the application to see your konnector connected.